/*
  Mini-station météo Arduino

  Version: 1.0
  Auteur: Jelora
  Date: 22/10/2024
  Licence: Creative Commons BY-NC-SA
*/

#include <Adafruit_BME280.h>
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>

#define adresseCapteurInterieur 0x76
#define adresseCapteurExterieur 0x77

#define LCD_RST 2
#define LCD_CE 3
#define LCD_DC 4
#define LCD_DIN 5
#define LCD_CLK 6

Adafruit_BME280 capteurInterieur;
Adafruit_BME280 capteurExterieur;

// CLR - Data clock
// DIN = Data IN
// DC = Data/Command select
// CE = Chip Enable
// RST = Reset
Adafruit_PCD8544 display = Adafruit_PCD8544(LCD_CLK, LCD_DIN, LCD_DC, LCD_CE, LCD_RST);

void setup() {
  Serial.begin(9600);
  while(!Serial);
  Serial.println();

  Serial.print("Initialisation BME280 intérieur -> ");
  if( !capteurInterieur.begin(adresseCapteurInterieur) ) {
    Serial.println("ERREUR");
  }
  else {
    Serial.println("OK");
  }

  Serial.print("Initialisation BME280 extérieur -> ");
  if( !capteurExterieur.begin(adresseCapteurExterieur) ) {
    Serial.println("ERREUR");
  }
  else {
    Serial.println("OK");
  }

  display.begin();
  display.setContrast(57);
  display.setTextSize(1);

  display.clearDisplay();
}

void loop() {
  display.clearDisplay();
  
  // Intérieur
  capteurInterieur.init();
  if ( !isnan(capteurInterieur.readTemperature()) && capteurExterieur.readPressure() > 0 ){
    Serial.print("[INT] Temp = ");
    Serial.print(capteurInterieur.readTemperature());
    Serial.print(" °C / Humidité = ");
    Serial.print(capteurInterieur.readHumidity());
    Serial.print(" % / Pression = ");
    Serial.print(capteurInterieur.readPressure() / 100.0F);
    Serial.println(" hPa");
      
    display.setCursor(12, 2);
    display.write("INT");
    display.drawRect(11, 0, 20, 11, BLACK);

    // Température
    afficherTemperature(false, capteurInterieur.readTemperature(), 15, 30);
    // Humidité
    afficherHumidite(false, capteurInterieur.readHumidity(), 0, 100);
  }

  // Extérieur
  capteurExterieur.init();
  if ( !isnan(capteurExterieur.readTemperature()) && capteurExterieur.readPressure() > 0 ){
    Serial.print("[EXT] Temp = ");
    Serial.print(capteurExterieur.readTemperature());
    Serial.print(" °C / Humidité = ");
    Serial.print(capteurExterieur.readHumidity());
    Serial.print(" % / Pression = ");
    Serial.print(capteurExterieur.readPressure() / 100.0F);
    Serial.println(" hPa");

    display.setCursor(55, 2);
    display.write("EXT");
    display.drawRect(53, 0, 21, 11, BLACK);

    // Température
    afficherTemperature(true, capteurExterieur.readTemperature(), 0, 40);
    // Humidité
    afficherHumidite(true, capteurExterieur.readHumidity(), 0, 100);
  }

  // Récupération de la pression atmosphérique depuis le capteur extérieur
  if ( !isnan(capteurExterieur.readPressure()) && capteurExterieur.readPressure() > 0 ){
    afficherPressionAtmospherique(capteurExterieur.readPressure() / 100.0F);
  }
  // ou depuis le capteur intérieur si le capteur extérieur est en panne
  else if ( !isnan(capteurInterieur.readPressure()) && capteurInterieur.readPressure() > 0 ){
    afficherPressionAtmospherique(capteurInterieur.readPressure() / 100.0F);
  }


  // Message d'erreur si aucun capteur fonctionnel
  if ( isnan(capteurInterieur.readTemperature()) || capteurInterieur.readTemperature() > 100
        || isnan(capteurExterieur.readTemperature()) || capteurExterieur.readTemperature() > 100 ){
    display.setCursor(0, 0);
    display.write("Aucun capteur");
    display.setCursor(0, 12);
    display.write("fonctionnel...");
  }

  display.display();

  // Refresh toutes les 5 secondes
  delay(5000);
}

void afficherTemperature(bool temperatureExterieure, float temperature, float minBar, float maxBar){
  int declageGraphique = 0;
  if ( temperatureExterieure ){
    declageGraphique = 43;
  }

  char temperatureString[5];
  dtostrf(temperature, 1, 1, temperatureString);
  if ( temperature >= 0 && temperature < 9.95 ){ // 9.95 au lieu de 10 car à l'affichage 9.95-9.99 est arrondi et affiché 10.0 au lieu de 9.9
    // Température 0.0 à 9.9°C
    // sur 3 caractères
    display.setCursor(declageGraphique+6, 13);
    display.write(temperatureString);
    display.setCursor(declageGraphique+30, 13);
    display.write("C");
    display.drawCircle(declageGraphique+27, 14, 1, BLACK);
  }
  else if ( temperature <= -9.95 ){ 
    // Température de -99.9 à -10.0
    // sur 5 caractères
    display.setCursor(declageGraphique+0, 13);
    display.write(temperatureString);
    display.setCursor(declageGraphique+35, 13);
    display.write("C");
    display.drawCircle(declageGraphique+32, 14, 1, BLACK);
  }
  else{
    // Température de 10.0 à 99.9°C et -9.9 à -0.1
    // sur 4 caractères
    display.setCursor(declageGraphique+3, 13);
    display.write(temperatureString);
    display.setCursor(declageGraphique+33, 13);
    display.write("C");
    display.drawCircle(declageGraphique+30, 14, 1, BLACK);
  }
  
  // Barre
  afficherBar(declageGraphique, 21, 41, temperature, minBar, maxBar);
}

void afficherHumidite(bool humiditeExterieure, float humidite, float minBar, float maxBar){
  int declageGraphique = 0;
  if ( humiditeExterieure ){
    declageGraphique = 43;
  }

  char humiditeString[3];
  dtostrf(humidite, 1, 0, humiditeString);
  if ( humidite == 100 ){
    // Humidite à 100 %
    // sur 3 caractères
    display.setCursor(declageGraphique+8, 25);
    display.write("100");
    display.setCursor(declageGraphique+29, 25);
    display.write("%");
  }
  else if ( humidite >= 9.5 && humidite < 100 ){ // 9.5 au lieu de 10 car à l'affichage 9.5 est arrondi et affiché 10
    // Humidite entre 10 et 99%
    // sur 2 caractères
    display.setCursor(declageGraphique+11, 25);
    display.write(humiditeString);
    display.setCursor(declageGraphique+26, 25);
    display.write("%");
  }
  else if ( humidite >= 0 && humidite < 9.5 ){
    // Humidite entre 0 et 9%
    // sur 1 caractère
    display.setCursor(declageGraphique+14, 25);
    display.write(humiditeString);
    display.setCursor(declageGraphique+22, 25);
    display.write("%");
  }
  else{
    // Humidité en dessous de 0% ou supérieure à 100% ?
    // Pas possible
  }

  // Barre
  afficherBar(declageGraphique, 33, 41, humidite, minBar, maxBar);
}

void afficherBar(int colonne, int ligne, float tailleBar, float valeur, float minBar, float maxBar){
  int valeurBar = 0;
  if ( valeur > minBar && valeur < maxBar ){
    valeurBar = ((valeur - minBar) / (maxBar - minBar)) * (tailleBar - 2);
  }
  if ( valeur >= maxBar ){
    valeurBar = 39;
  }
  display.drawRect(colonne+0, ligne, tailleBar, 3, BLACK);
  if ( valeurBar > 0 ){
    display.drawLine(colonne+1, ligne+1, colonne+valeurBar, ligne+1, BLACK);
  }
}

void afficherPressionAtmospherique(float pression){
  // Pression atmosphérique en bas
  char pressionString[4];
  dtostrf(pression, 1, 0, pressionString);
  if ( pression > 1000 ){
    display.setCursor(19, 37);
    display.write(pressionString);
    display.setCursor(46, 37);
    display.write("hPa");
  }
  else{
    display.setCursor(24, 37);
    display.write(pressionString);
    display.setCursor(45, 37);
    display.write("hPa");
  }
  afficherBar(0, 45, 84, pression, 970, 1056);
}
